<?php
namespace Wovnio\Wovnwp;
require_once( 'constants.php' );

class WovnSettingsPage
{
		/**
		 * Holds the values to be used in the fields callbacks
		 */
		private $options;

		/**
		 * Start up
		 */
		public function __construct()
		{
				add_action( 'admin_menu', array( $this, 'add_plugin_page' ) );
				add_action( 'admin_init', array( $this, 'page_init' ) );
				add_action( 'update_option_wovn_options', array( $this, 'flush_rewrite_rules' ), 10, 3 );
				add_action( 'update_option_permalink_structure', array( $this, 'update_permalink_structure_callback' ), 10, 3 );
 				wp_enqueue_style( 'wovn-css', plugins_url( 'css/styles.css', __FILE__ ), array() );
		}

		/**
		 * Add options page
		 */
		public function add_plugin_page()
		{
				// This page will be under "Settings"
				add_options_page(
						'Wovn Settings',
						'WOVN.io',
						'manage_options',
						'wovn-setting-admin',
						array( $this, 'create_admin_page' )
				);
		}

		/**
		 * Options page callback
		 */
		public function create_admin_page()
		{
				// Set class property
				$this->options = get_option( 'wovn_options' );
				?>
				<div class="wrap wovn-plugin-admin">
						<?php screen_icon(); ?>
						<h2><img src="<?php echo plugins_url( 'images/header-logo.svg', __FILE__ ); ?>" alt="WOVN.io"> WOVN.io</h2>
						<p id="wovn-registration-text">For membership registration and translation settings please go to the <a href="https://wovn.io/" target="_blank">WOVN.io official website.</a><p>
						<div class="form-table-wrapper wovn-plugin-table">
							<form method="post" action="options.php">
							<?php
									// This prints out all hidden setting fields
									settings_fields( 'wovn_option_group' );
									do_settings_sections( 'wovn-setting-admin' );
									submit_button();
							?>
							</form>
						</div>
				</div>
				<?php
		}

		/**
		 * Register and add settings
		 */
		public function page_init()
		{
				register_setting(
						'wovn_option_group', // Option group
						'wovn_options', // Option name
						array( $this, 'sanitize' ) // Sanitize
				);

				add_settings_section(
						'setting_section_id', // ID
						'', // Title
						array( $this, 'print_section_info' ), // Callback
						'wovn-setting-admin' // Page
				);

				// user_token
				add_settings_field(
						'wovn_user_token',
						'<i class=" dashicons-before dashicons-admin-network"></i>&nbsp;&nbsp;Project Token',
						array( $this, 'wovn_user_token_callback' ),
						'wovn-setting-admin',
						'setting_section_id'
				);

				// default_lang
				add_settings_field(
						'wovn_default_lang',
						'<i class=" dashicons-before dashicons-admin-site"></i>&nbsp;&nbsp;Default Language',
						array( $this, 'wovn_default_lang_callback' ),
						'wovn-setting-admin',
						'setting_section_id'
				);

				// url_pattern_name
				add_settings_field(
						'wovn_url_pattern_name',
						'<i class=" dashicons-before dashicons-admin-links"></i>&nbsp;&nbsp;Url Pattern Name',
						array( $this, 'wovn_url_pattern_name_callback' ),
						'wovn-setting-admin',
						'setting_section_id'
				);

				// supported_langs
				add_settings_field(
						'wovn_supported_langs',
						'<i class=" dashicons-before dashicons-translation"></i>&nbsp;&nbsp;Supported Languages',
						array( $this, 'wovn_supported_langs_callback' ),
						'wovn-setting-admin',
						'setting_section_id'
				);
		}

		/**
		 * Load our custom CSS sheet.
		 */
		function load_custom_admin_style() {
			wp_enqueue_style( 'wovn', plugins_url( 'css/styles.css', __FILE__ ), array() );
		}

		/**
		 * Sanitize each setting field as needed
		 *
		 * @param array $input Contains all settings fields as array keys
		 */
		public function sanitize( $input )
		{
				$new_input = $input;
				if( isset( $input['wovn_user_token'] ) )
						$new_input['wovn_user_token'] = sanitize_text_field( $input['wovn_user_token'] );

				return $new_input;
		}

		/**
		 * Print the Section text
		 */
		public function print_section_info()
		{

		}

		/**
		 * Get the settings option array and print one of its values
		 */
		public function wovn_user_token_callback()
		{
				printf(
						'<input type="text" id="wovn_user_token" name="wovn_options[wovn_user_token]" value="%s" />',
						isset( $this->options['wovn_user_token'] ) ? esc_attr( $this->options['wovn_user_token']) : ''
				);
		}

		/**
		 * Get the settings option array and print one of its values
		 */
		public function wovn_default_lang_callback()
		{
				echo '<select id="wovn_default_lang" name="wovn_options[wovn_default_lang]">';
				foreach(array_values(Constants::$langs) as $lang) {
					printf(
						'<option value="%s" %s>%s</option>',
						$lang['code'],
						selected( $this->options['wovn_default_lang'], $lang['code'] ),
						$lang['name']
					);
				}
				print '</select>';
		}

		/**
		 * Get the settings option array and print one of its values
		 */
		public function wovn_url_pattern_name_callback()
		{
			$permalink_structure = get_option( 'permalink_structure' );
			$url_pattern_names = Constants::$url_pattern_names;
			echo '<select id="wovn_url_pattern_name" name="wovn_options[wovn_url_pattern_name]">';
			foreach($url_pattern_names as $name) {
				printf(
					'<option value="%s" %s %s>%s</option>',
					$name,
					selected( $this->options['wovn_url_pattern_name'], $name ),
					(empty($permalink_structure) && $name == 'path') ? 'disabled="disabled"' : '',
					$name
				);
			}
			echo '</select>';
			if (empty($permalink_structure)) {
				echo('<br>Please change your pemalink settings if you\'d like to set url pattern name to "<b>path</b>"');
			}
		}

		/**
		 * Get the settings option array and print one of its values
		 */
		public function wovn_supported_langs_callback()
		{
			?>
				<table class="wp-list-table widefat wovn-lang-table">
					<thead>
						<tr>
							<th>Code</th>
							<th>Flag</th>
							<th>Orginal name</th>
							<th>English name</th>
							<th>Enabled</th>
						</tr>
					</thead>
					<tbody>
			<?php
			foreach(array_values(Constants::$langs) as $lang) {
				$checked = isset( $this->options['wovn_supported_langs'] ) ? in_array($lang['code'], $this->options['wovn_supported_langs']) : false;

				echo '<tr>';
				echo '<td>' . $lang['code'] . '</td>';
				echo '<td><img src="' . plugins_url( 'images/flags/' . $lang['code'] . '.png', __FILE__ ) . '" alt="' . $lang['name'] . ' flag"</td>';
				echo '<td>' . $lang['name'] . '</td>';
				echo '<td>' . $lang['en'] . '</td>';
				printf(
					'<td><input type="checkbox" id="' . $lang['code'] . '" name="wovn_options[wovn_supported_langs][]" value="%s" %s /></td>',
					$lang['code'],
					$checked ? 'checked="checked"' : ''
				);
				echo '</tr>';
			}
			?>
				</tbody>
				<tfoot>
					<th>Code</th>
					<th>Flag</th>
					<th>Orginal name</th>
					<th>English name</th>
					<th>Enabled</th>
				</tfoot>
			</table>
			<?php
		}

		// after save options, flush rewrite rules
		public function flush_rewrite_rules( $old_value, $new_value ){
			global $wp_rewrite;
			$wp_rewrite->flush_rules();
		}

		// Url_pattern_name 'path' is not possible with Default permalinks.
		public function update_permalink_structure_callback( $old_value, $new_value ) {
			// Set class property
			$this->options = get_option( 'wovn_options' );
			$permalink_structure = get_option( 'permalink_structure' );
			if (empty($permalink_structure) && $this->options['wovn_url_pattern_name'] == 'path') {
				$this->options['wovn_url_pattern_name'] = 'query';
				update_option( 'wovn_options', $this->options );
			}
		}
}

if( is_admin() )
		new WovnSettingsPage();
